#!/usr/bin/env python

#
# Medusa server that exposes the skeleton program.
#

from optparse import OptionParser

try:
    from medusa import http_server, xmlrpc_handler
except ImportError:
    print '\n' + '='*50
    print 'medusa/ package not found; fetch it from '
    print '  http://www.amk.ca/files/python/'
    print 'and install it.'
    print '='*50
    raise

import asyncore
from quixote.publish import Publisher
from quixote.server import medusa_http
from pyPgSQL import PgSQL
import xmlrpclib

from quixote import enable_ptl
enable_ptl()

# Warning: Must import SOAPpy *after* enable_ptl.  I don't know why.
from SOAPpy import SOAPProxy


# Default port to listen on.
PORT = 8081

import connection_args
CONNECTION_ARGS = connection_args.connection_args
##CONNECT_ARGS_FILE = 'connection_args.txt'

DEBUG = 0

def dbgmsg(*args):
    if DEBUG:
        for arg in args:
            print arg,
        print


#
# A container for persistent connections etc.
#
class PersistentContainer:
    pass


class MyPublisher(Publisher):
    def __init__(self, root_namespace, config=None):
        Publisher.__init__(self, root_namespace, config)
        self.container = PersistentContainer()
##        connectionFile = file(CONNECT_ARGS_FILE, 'r')
##        connectionArgs = connectionFile.read().strip()
##        connectionFile.close()
##        self.container.dbConnection = PgSQL.connect(connectionArgs)
        self.container.dbConnection = PgSQL.connect(**CONNECTION_ARGS)
        self.container.xmlrpcProxy = xmlrpclib.ServerProxy('http://localhost:8082')
        self.container.soapProxy = SOAPProxy(str("http://localhost:8083/"))
        meerkatURI = "http://www.oreillynet.com/meerkat/xml-rpc/server.php"
        self.container.meerkatServer = xmlrpclib.Server(meerkatURI)
        dbgmsg('1a. (MyPublisher.__init__) created dbConnection:', self.container.dbConnection)
        dbgmsg('1b. (MyPublisher.__init__) created XML-RPC proxy:', self.container.xmlrpcProxy)
        dbgmsg('1c. (MyPublisher.__init__) created SOAP proxy:', self.container.soapProxy)
        dbgmsg('1d. (MyPublisher.__init__) created Meerkat server:', self.container.meerkatServer)

    def __del__(self):
        self.container.dbConnection.close()
        print '3. (MyPublisher.__del__) closed dbConnection:', self.container.dbConnection

    def cleanup(self):
        self.container.dbConnection.close()
        print '3. (MyPublisher.cleanup) closed dbConnection:', self.container.dbConnection

    def start_request(self, request):
        Publisher.start_request(self, request)
        request.container = self.container
##        request.dbConnection = self.dbConnection
##        request.xmlrpcProxy = self.xmlrpcProxy
##        request.soapProxy = self.soapProxy
##        request.meerkatServer = self.meerkatServer
        dbgmsg('2a. (MyPublisher.start_request) Passing db con:', self.container.dbConnection)
        dbgmsg('2b. (MyPublisher.start_request) Passing XML-RPC proxy:', self.container.xmlrpcProxy)
        dbgmsg('2c. (MyPublisher.start_request) Passing SOAP proxy:', self.container.soapProxy)
        dbgmsg('2d. (MyPublisher.start_request) Passing Meerkat server:', self.container.meerkatServer)


def start_server(port):
    print 'Now serving the skeleton services on port %i' % port
    server = http_server.http_server('', port)
    publisher = MyPublisher('skeleton.ui')
    publisher.read_config('config.py')
    # Can also set the log names in config.py.
    #publisher.config.debug_log = "/tmp/debug"
    #publisher.config.error_log = "/tmp/error"
    publisher.setup_logs()
    #print 'publisher.container:', publisher.container
    dh = medusa_http.QuixoteHandler(publisher, 'skeleton/services on medusa server', server)
    server.install_handler(dh)
    try:
        asyncore.loop()
    #except KeyboardInterrupt:
    except:
        publisher.cleanup()


USAGE_TEXT = """
    python %prog [options] <infile.rng>
example:
    python %prog infile.rng"""

def usage(parser):
    parser.print_help()
    sys.exit(-1)


def main():
    global Verbose
    parser = OptionParser(USAGE_TEXT)
    parser.add_option("-p", "--port", type="int",
        dest="port", help="port to listen on")
##    parser.add_option("-f", "--search_files", action="store_true",
##        dest="search_files", help="search a set of files")
##    parser.add_option("-u", "--search_url", action="store_true",
##        dest="search_url", help="retrieve and search a url")
##    parser.add_option("-o", "--options", type="string", action="store",
##        dest="options", help="additional options/flags to pass to sgrep")
##    parser.add_option("-f", "--file", type="string", dest="filename",
##        help="read data from FILENAME")
##    parser.add_option("-v", "--verbose", 
##        action="store_true", dest="verbose")
##     parser.add_option("-q", "--quiet",
##         action="store_false", dest="verbose")
##    parser.add_option("-f", "--file", type="string", dest="filename",
##        help="read data from FILENAME")
##    parser.add_option("-v", "--verbose", 
##        action="store_true", dest="verbose")
##    parser.add_option("-m", "--list_methods", action="store_true",
##        dest="list_methods", help="print list of methods")
##    parser.add_option("-p", "--port", type="int",
##        dest="port")
##     parser.add_option("-q", "--quiet",
##         action="store_false", dest="verbose")
    (options, args) = parser.parse_args()
    port = PORT
    if options.port:
        port = options.port
    if len(args) == 0:
        start_server(port)
    else:
        usage(parser)

        
if __name__ == "__main__":
    main()
    #import pdb
    #pdb.run('main()')


