# Make this directory a package.

_q_exports = [
    'soap_source',
    'skeleton_dir',
    'dave_html',
##    'pyfav_gif',
    ]


import time
from quixote.util import StaticFile, StaticDirectory
from servicesui import MenuUI, InfoUI, ServicesUI, MeerkatUI, ZipfileUI, TestsUI


def _q_lookup(request, name):
    # Test: Stall long enough so that we can make several requests and force the creation
    #   of more than one child process.
    if name == 'info':
        # Process requests whose URL is: http://host:8081/info/do_xxxx.
        handler = InfoHandler()
        return handler
    elif name == 'service':
        # Process requests whose URL is: http://host:8081/service/do_xxxx.
        handler = ServiceHandler()
        return handler
    elif name == 'meerkat':
        # Process requests whose URL is: http://host:8081/meerkat/do_xxxx.
        handler = MeerkatHandler()
        return handler
    elif name == 'zipfile':
        # Process requests whose URL is: http://host:8081/zipfile/do_xxxx.
        handler = ZipfileHandler()
        return handler
    elif name == 'tests':
        # Process requests whose URL is: http://host:8081/tests/do_xxxx.
        handler = TestsHandler()
        return handler
##    elif name == 'skeleton_dir':
##        print '(_q_lookup) handling skeleton_dir'
##        return skeleton_dir
    else:
        menu = MenuUI()
        return menu.index(request)


#
# Process requests whose URL is: http://host:8081/info/do_xxxx.
# If the request URL ends in ../requestinfo or ../serverinfo, then call
#   InfoUI.do_requestinfo or InfoUI.do_serverinfo.
#
class InfoHandler:
    _q_exports = ['requestinfo', 'serverinfo']
    def requestinfo(self, request):
        info = InfoUI()
        content = info.do_requestinfo(request)
        return content
    def serverinfo(self, request):
        info = InfoUI()
        content = info.do_serverinfo(request)
        return content
    def _q_index(self, request):
        menu = MenuUI()
        return menu.index(request)


#
# Process requests whose URL is: http://host:8081/service/do_xxxx.
# If class ServicesUI contains a method 'do_+name, call it.
#
class ServiceHandler:
    _q_exports = []
    def _q_lookup(self, request, name):
        services = ServicesUI()
        meth = getattr(services, 'do_' + name, None)
        if meth:
            return meth(request)
        else:
            menu = MenuUI()
            return menu.index(request)


#
# Process requests whose URL is: http://host:8081/meerkat/do_xxxx.
# If class MerrkatUI contains a method 'do_+name, call it.
#
class MeerkatHandler:
    _q_exports = []
    def _q_lookup(self, request, name):
        meerkat = MeerkatUI()
        meth = getattr(meerkat, 'do_' + name, None)
        if meth:
            return meth(request)
        elif name == 'items':
            return meerkat
        else:
            menu = MenuUI()
            return menu.index(request)


#
# Process requests whose URL is: http://host:8081/zipfile/do_xxxx.
# If class ZipfileUI contains a method 'do_+name, call it.
#
class ZipfileHandler:
    _q_exports = []
    def _q_lookup(self, request, name):
        ui = ZipfileUI()
        meth = getattr(ui, 'do_' + name, None)
        if meth:
            return meth(request)
        else:
            menu = MenuUI()
            return menu.index(request)


#
# Process requests whose URL is: http://host:8081/tests/do_xxxx.
# If class TestsUI contains a method 'do_+name, call it.
#
class TestsHandler:
    _q_exports = []
    def _q_lookup(self, request, name):
        ui = TestsUI()
        meth = getattr(ui, 'do_' + name, None)
        if meth:
            return meth(request)
        else:
            menu = MenuUI()
            return menu.index(request)


#
# Expose various static files and directories.
#
soap_source = StaticFile(
    "/w1/Python/Quixote/skeleton/soapserver.py",
    follow_symlinks=1,
    mime_type="text/plain"
    )

dave_html = StaticFile(
    "/w1/Python/Quixote/skeleton/dave.html",
    follow_symlinks=1,
    mime_type="text/html"
    )
##
##pyfav_gif = StaticFile(
##    "/w1/Python/Quixote/skeleton/pyfav.gif",
##    follow_symlinks=1,
##    mime_type="application/gif"
##    )
##
###
### Notes: 
### - In order for StaticDirectory to work, there must be a slash on the
###   end of the URI.
### - Without the list_directory=1 argument, the files in the directory
###   are available, but the directory is not shown.
###

skeleton_dir = StaticDirectory("/w1/Python/Quixote/skeleton", list_directory=1)

##pyfav_gif = StaticFile("/w1/Python/Quixote/skeleton/pyfav.gif", mime_type="application/gif")

